/***********************************************************************
SOFTWARE RELEASE: MarMOT 1.1.1 DATE: 23 January 2015 LICENCE: CeCILL 2.1

Includes modifications to 2013 MarMOT 1.1 code (MarMOT 1.1.0) as listed 
***********************************************************************/
/***********************************************************************

SOFTWARE RELEASE: MarMOT 1.1  DATE: 21 November 2013

Copyright Natural Environment Research Council, UK

Author: John Hemmings, National Oceanography Centre 
e-mail: j.hemmings@noc.ac.uk (until 30 Nov 2013) 
Alternative e-mail for author: marmot@wessexenv.mail1.co.uk 

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

***********************************************************************/
/***********************************************************************

 optimizer.h - John Hemmings - 24 Oct 2012 

 MODIFIED:

 John Hemmings - 08 Jan 2015 
 - Removed default definition of OBJF_FREEPARMTRANSFORM_REQ (now in gfan.h)
 - Removed now-redundant section of 'gadatarec' structure 
 - Removed function prototypes that are now declared elsewhere

***********************************************************************/

#include "gfan.h"


/* G.A. optimizer configuration data & work area */

struct gadatarec {

  double (*func)(struct gadatarec *, OBJF_FLOAT_TYPE []);
                                      /* Objective function driver */
  struct gactrlrec *ctrl;             /* G.A. configuration data */
  FILE *logfile;                      /* G.A. log file */
  struct parmctrlrec *parmctrl;       /* Free parameter space configuration */

                                      /* -- G.A. INFO FOR FUNCTION DRIVER -- */
  double *knownfitness;               /* Pointer to a previously determined 
                                         fitness for the current individual 
                                         if available */

                                      /* -- COUNTERS FOR REPORTING -- */
  int gennum;                         /* Current generation number */
  int psetnum;                        /* Current individual number */ 

                                      /* -- FUNCTION/DRIVER SPECIFIC DATA -- */
  struct casetabrec *funcdata;        /* Objective function data */
  int ipset;                          /* Index to parameter set in fn. data */
  FILE *funcoutfile[N_OUTTAB_NZ];     /* Objective function output files */
  FILE *evalfile;                     /* G.A. fn. evaluation output file */
  clock_t startcpu;                   /* Start time for G.A. optimization */
  char **parmname;                    /* Parameter names */
};


/* Powell optimizer configuration data & work area */

struct powelldatarec {

  double (*func)(struct powelldatarec *, double []);
                                      /* Objective function driver */
  struct powellctrlrec *ctrl;         /* Powell configuration data */ 
  FILE *logfile;                      /* Powell log file */
  struct parmctrlrec *parmctrl;       /* Free parameter space configuration */
  int n;                              /* Dimensionality of parameter space */

                                      /* -- WORK AREA -- */
  double **direction;                 /* Workspace for direction set */
  double *dir;                        /* Workspace for new direction */
  double *p0;                         /* Workspace for parameter vector */
  double *pext;                       /* Workspace for parameter vector */
  double *parmval;                    /* Workspace for parameter vector */

                                      /* -- COUNTERS FOR REPORTING -- */
  int iternum;                        /* Current Powell iteration number */
  int dirnum;                         /* Current direction number */
  int evalnum;                        /* Current function evaluation number */

                                      /* -- FUNCTION/DRIVER SPECIFIC DATA -- */
  struct casetabrec *funcdata;        /* Objective function data */
  int ipset;                          /* Index to parameter set in fn. data */
  FILE *funcoutfile[N_OUTTAB_NZ];     /* Objective function output files */
  FILE *evalfile;                     /* Powell fn. evaluation output file */
  int psetno;                         /* Parameter set id number */
  clock_t startcpu;                   /* Start time for Powell minimization */
  int funccount;                      /* Tot. number of function evaluations */
};


/* Function prototypes */

int minimize_ga (struct gadatarec *, OBJF_FLOAT_TYPE *, double [], int *); 
int minimize_powell (struct powelldatarec *, double p[], double *, char **);

