!**************************************************************************!
! SOFTWARE RELEASE: MarMOT 1.1.1 DATE: 23 January 2015 LICENCE: CeCILL 2.1 !
!**************************************************************************!
!**************************************************************************!
!                                                                          ! 
! SOFTWARE RELEASE: MarMOT 1.1  DATE: 21 November 2013                     ! 
!                                                                          !
! Copyright NEMO Consortium                                                !
!                                                                          !
! Author: John Hemmings, National Oceanography Centre                      !
! e-mail: j.hemmings@noc.ac.uk (until 30 Nov 2013)                         !
! Alternative e-mail for author: marmot@wessexenv.mail1.co.uk              !
!                                                                          !
! Code imported from trcopt_medusa.F90 was written by other authors.       !
! See imported header for details.                                         !   
!                                                                          !
! Author: John Hemmings, National Oceanography Centre                      !
! e-mail: j.hemmings@noc.ac.uk (until 30 Nov 2013)                         !
! Alternative e-mail for author: marmot@wessexenv.mail1.co.uk              !
!                                                                          !
! This software is governed by the CeCILL  license under French law and    !
! abiding by the rules of distribution of free software.  You can  use,    ! 
! modify and/ or redistribute the software under the terms of the CeCILL   !
! license as circulated by CEA, CNRS and INRIA at the following URL        !
! "http://www.cecill.info".                                                ! 
!                                                                          !
! As a counterpart to the access to the source code and  rights to copy,   !
! modify and redistribute granted by the license, users are provided only  !
! with a limited warranty  and the software's author,  the holder of the   !
! economic rights,  and the successive licensors  have only  limited       !
! liability.                                                               ! 
!                                                                          !
! In this respect, the user's attention is drawn to the risks associated   !
! with loading,  using,  modifying and/or developing or reproducing the    !
! software by the user in light of its specific status of free software,   !
! that may mean  that it is complicated to manipulate,  and  that  also    !
! therefore means  that it is reserved for developers  and  experienced    !
! professionals having in-depth computer knowledge. Users are therefore    !
! encouraged to load and test the software's suitability as regards their  !
! requirements in conditions enabling the security of their systems and/or ! 
! data to be ensured and,  more generally, to use and operate it in the    ! 
! same conditions as regards security.                                     ! 
!                                                                          ! 
! The fact that you are presently reading this means that you have had     !
! knowledge of the CeCILL license and that you accept its terms.           !
!                                                                          !
!**************************************************************************!
!===============================================================================
! MODULE medusa_opt 
! John Hemmings - 18 Jun 2012
!
! Contains a version of the NEMO subroutine 'trc_opt_medusa' of Yool et al
! (2011), adapted for use with MarMOT.
!
! Original code modified to copy data to MarMOT diagnostic variables. 
! N.B. Level 'jpk' may now be part of water column. 
!
! Reference:
! Yool, A., Popova, E.E., and Anderson, T.R., 2011. Medusa-1.0: a new 
! intermediate complexity plankton ecosystem model for the global domain, 
! Geosci. Model Dev., 4, 381–417, doi:10.5194/gmd-4-381-2011.
!===============================================================================

!******************* Imported code [trcopt_medusa.F90] *************************
   !!======================================================================
   !!                         ***  MODULE trcopt_medusa  ***
   !! TOP :   MEDUSA Compute the light availability in the water column
   !!======================================================================
   !! History :    -   !  1995-05  (M. Levy) Original code
   !!              -   !  1999-09  (J.-M. Andre, M. Levy) 
   !!              -   !  1999-11  (C. Menkes, M.-A. Foujols) itabe initial
   !!              -   !  2000-02  (M.A. Foujols) change x**y par exp(y*log(x))
   !!             2.0  !  2007-12  (C. Deltel, G. Madec)  F90
   !!              -   !  2008-08  (K. Popova) adaptation for MEDUSA
   !!              -   !  2008-11  (A. Yool) continuing adaptation for MEDUSA
   !!              -   !  2010-03  (A. Yool) updated for branch inclusion
   !!----------------------------------------------------------------------
   !!----------------------------------------------------------------------
   !!   'key_medusa'                                      MEDUSA bio-model
   !!----------------------------------------------------------------------
   !!   trc_opt_medusa        :   Compute the light availability in the water column
   !!----------------------------------------------------------------------
!************************** End of imported code *******************************

  MODULE medusa_opt

  USE medusa_globdata 

  IMPLICIT NONE
  PRIVATE

  PUBLIC trc_opt_medusa

!******************* Imported code [trcopt_medusa.F90] *************************
   !!----------------------------------------------------------------------
   !! NEMO/TOP 2.0 , LOCEAN-IPSL (2007) 
   !! $Id: trcopt_medusa.F90 1146 2008-06-25 11:42:56Z rblod $ 
   !! Software governed by the CeCILL licence (modipsl/doc/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------

CONTAINS

   SUBROUTINE trc_opt_medusa( kt )
      !!---------------------------------------------------------------------
      !!                     ***  ROUTINE trc_opt_medusa  ***
      !!
      !! ** Purpose :   computes the light propagation in the water column
      !!              and the euphotic layer depth
      !!
      !! ** Method  :   local par is computed in w layers using light propagation
      !!              mean par in t layers are computed by integration
      !!---------------------------------------------------------------------
      INTEGER, INTENT( in ) ::   kt   ! index of the time stepping
      INTEGER  ::   ji, jj, jk
      REAL(wp) ::   zpig                                    ! total pigment
      REAL(wp) ::   zkr                                     ! total absorption coefficient in red
      REAL(wp) ::   zkg                                     ! total absorption coefficient in green
      REAL(wp) ::   totchl                                  ! total Chl concentreation
      REAL(wp), DIMENSION(jpi,jpj)     ::   zpar100         ! irradiance at euphotic layer depth
      REAL(wp), DIMENSION(jpi,jpj)     ::   zpar0m          ! irradiance just below the surface
!************************** End of imported code *******************************
      REAL(wp), DIMENSION(jpi,jpj,jpk+1) :: zparr, zparg    ! red and green compound of par
!******************* Imported code [trcopt_medusa.F90] *************************

      CHARACTER (len=25) :: charout
      !!---------------------------------------------------------------------

      IF( kt == nit000 ) THEN
         IF(lwp) WRITE(numout,*)
         IF(lwp) WRITE(numout,*) ' trc_opt_medusa: MEDUSA optic-model'
         IF(lwp) WRITE(numout,*) ' ~~~~~~~'
      ENDIF

      ! determination of surface irradiance
      ! -----------------------------------
      zpar0m (:,:)   = qsr   (:,:) * 0.43
      zpar100(:,:)   = zpar0m(:,:) * 0.01
      xpar   (:,:,1) = zpar0m(:,:)
      zparr  (:,:,1) = 0.5 * zpar0m(:,:)
      zparg  (:,:,1) = 0.5 * zpar0m(:,:)


      ! determination of xpar
      ! ---------------------

!************************** End of imported code *******************************
      DO jk = 2, jpk+1                 ! determination of local par in w levels
!******************* Imported code [trcopt_medusa.F90] *************************
         DO jj = 1, jpj
            DO ji = 1, jpi
               totchl =trn(ji,jj,jk-1,jpchn)+trn(ji,jj,jk-1,jpchd)
               zpig = MAX( TINY(0.), totchl/rpig) 
               zkr  = xkr0 + xkrp * EXP( xlr * LOG( zpig ) )
               zkg  = xkg0 + xkgp * EXP( xlg * LOG( zpig ) )
               zparr(ji,jj,jk) = zparr(ji,jj,jk-1) * EXP( -zkr * fse3t(ji,jj,jk-1) )
               zparg(ji,jj,jk) = zparg(ji,jj,jk-1) * EXP( -zkg * fse3t(ji,jj,jk-1) )
            END DO
        END DO
      END DO
!************************** End of imported code *******************************

      zparr_top_diag=zparr(jpi,jpj,:)
      zparg_top_diag=zparg(jpi,jpj,:)

      DO jk = 1, jpk                     ! mean par in t levels
!******************* Imported code [trcopt_medusa.F90] *************************
         DO jj = 1, jpj
            DO ji = 1, jpi
               totchl =trn(ji,jj,jk  ,jpchn)+trn(ji,jj,jk  ,jpchd)
               zpig = MAX( TINY(0.), totchl/rpig) 
               zkr  = xkr0 + xkrp * EXP( xlr * LOG( zpig ) )
               zkg  = xkg0 + xkgp * EXP( xlg * LOG( zpig ) )
               zparr(ji,jj,jk)    = zparr(ji,jj,jk) / zkr / fse3t(ji,jj,jk) * ( 1 - EXP( -zkr*fse3t(ji,jj,jk) ) )
               zparg(ji,jj,jk)    = zparg(ji,jj,jk) / zkg / fse3t(ji,jj,jk) * ( 1 - EXP( -zkg*fse3t(ji,jj,jk) ) )
               xpar (ji,jj,jk) = MAX( zparr(ji,jj,jk) + zparg(ji,jj,jk), 1.e-15 )
            END DO
         END DO
      END DO

      ! 3. Determination of euphotic layer depth
      ! ----------------------------------------

      ! Euphotic layer bottom level
      neln(:,:) = 1                                           ! initialisation of EL level
      DO jk = 1, jpk
         DO jj = 1, jpj
           DO ji = 1, jpi
              IF( xpar(ji,jj,jk) >= zpar100(ji,jj) )   neln(ji,jj) = jk+1 ! 1rst T-level strictly below EL bottom
              !                                                  ! nb. this is to ensure compatibility with
              !                                                  ! nmld_trc definition in trd_mld_trc_zint
           END DO
         END DO
      ENDDO

      ! Euphotic layer depth
      DO jj = 1, jpj
         DO ji = 1, jpi
            xze(ji,jj) = fsdepw( ji, jj, neln(ji,jj) )            ! exact EL depth
         END DO
      ENDDO 

!************************** End of imported code *******************************

  END SUBROUTINE trc_opt_medusa

  END MODULE medusa_opt 
