/***********************************************************************
SOFTWARE RELEASE: MarMOT 1.1.1 DATE: 23 January 2015 LICENCE: CeCILL 2.1
***********************************************************************/
/***********************************************************************

SOFTWARE RELEASE: MarMOT 1.1  DATE: 21 November 2013

Copyright Natural Environment Research Council, UK

Author: John Hemmings, National Oceanography Centre 
e-mail: j.hemmings@noc.ac.uk (until 30 Nov 2013) 
Alternative e-mail for author: marmot@wessexenv.mail1.co.uk 

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

***********************************************************************/

/* tabget.c - John Hemmings (Nov 2008) */

#include "toolbox.h"

#define INFILE 1 
#define OUTFILE 2 
#define VOFFSET 3


int main (int argc, char *argv[])

{ FILE *infile,*outfile;
  char *outfname;
  char outfbuff[FILENAME_MAX+1];
  char varlist[LINE_MAX+1];
  char *invarname[NFIELD_MAX];
  char *outvarname[2];
  char *reqvarname[NFIELD_MAX];
  int reqvar[NFIELD_MAX];
  static double x[NFIELD_MAX];
  static char fieldlist[LINE_MAX+1];
  static char *field[NFIELD_MAX];
  struct datarec d={x,fieldlist,field,NULL};
  static char fieldlist1[LINE_MAX+1];
  static char *field1[2];
  struct datarec dout={NULL,fieldlist1,field1,NULL};
  int maxlen[2],len;
  struct readctrlrec readctrl;
  int ninvar,nreqvar,status,i,j,ttbin,ttbout,recnum,numerr,lineno;

  version("1.00",VHDR,argc,argv);

  if (argc<2)
    { fprintf(stderr,"Usage: %s infile [outfile [recnum] [var1 ...]]\n",
              argv[0]);
      exit(EXIT_FAILURE);
    }

  for (i=1;i<argc;i++)
    if (strpbrk(argv[i],WHITESPACE)!=NULL) exiterror(BADARG,argv[0],argv[i],0);

  if (strcmp(argv[INFILE],"-")==0) infile=stdin;
  else if ((infile=fopen(argv[INFILE],"r"))==NULL)
    exiterror(OPENINERR,argv[0],argv[INFILE],0);

  status=readvar(infile,varlist,invarname,&ninvar,&ttbin,&lineno);
  if (status==OK_INLINE)
    { fclose(infile);
      infile=NULL;
    }
  else if (status!=OK) exiterror(status,argv[0],argv[INFILE],lineno);

  recnum=1;
  nreqvar=0;
  
  if (argc>3)
    { recnum=(int)(sgetnum(argv[VOFFSET],&numerr)+0.5);
      if (numerr || recnum<=0) 
        { recnum=1;
          j=VOFFSET;
         }
      else j=VOFFSET+1; 
      nreqvar=argc-j;
      if (nreqvar>NFIELD_MAX) exiterror(OUTVARERR,argv[0],NULL,0);
      for (i=0;i<nreqvar;i++) reqvarname[i]=argv[j+i];
    }

  if (nreqvar==0) 
    { nreqvar=ninvar;
      for (i=0;i<ninvar;i++)
        { reqvar[i]=i;
          reqvarname[i]=invarname[i];
        }
    }
  else if (indexlist(nreqvar,reqvarname,ninvar,invarname,reqvar)<nreqvar)
    { for (i=0;reqvar[i]>=0;i++);
      exiterror(VARERR,argv[0],reqvarname[i],0);
    }

  ttbout=0;
  if (argc<3) 
    { outfile=stdout;
      outfname="-";
    }
  else 
    { if ((outfile=outfopen(argv[OUTFILE],outfbuff,&outfname,&ttbout))==NULL)
        exiterror(OPENOUTERR,argv[0],outfname,0);
      if (ttbout) exiterror(BADEXPR,argv[0],argv[OUTFILE],0);
    }

  status=OK;
  for (i=1; i<recnum && status==OK; i++) 
    status=getdata(ninvar,NULL,infile,ttbin,NULL,invarname,&d,NULL,&lineno);
  if (status!=OK && status!=EOF) exiterror(status,argv[0],argv[INFILE],lineno);

  if (status!=EOF)
    { readctrl.nkeyvar=0;
      readctrl.keyvar=NULL;
      readctrl.nstrvar=ninvar;
      readctrl.strvar=NULL;
      status=getdata(ninvar,NULL,infile,ttbin,&readctrl,invarname,&d,NULL,
                     &lineno);
    }

  switch(status)
 
    { case OK:

        maxlen[0]=4;
        for (i=0;i<nreqvar;i++)
          { len=strlen(reqvarname[i]);
            if (len>maxlen[0]) maxlen[0]=len;
          }
        maxlen[1]=DEFAULT;

        if (argc<3 || strcmp(argv[OUTFILE],".")!=0)
          { outvarname[0]="NAME";
            outvarname[1]="VALUE";
            status=writevar(outfile,0,2,outvarname,maxlen,DEFAULT);
            if (status!=OK) exiterror(status,argv[0],outfname,0);
          }

        for (i=0;i<nreqvar;i++)
          { dout.field[0]=reqvarname[i];
            dout.field[1]=d.field[reqvar[i]];
            status=
              writedata(outfile,0,&dout,2,NULL,maxlen,NULL,DEFAULT,DEFAULT,1);
            if (status!=OK) exiterror(status,argv[0],outfname,0);
          }

        break;

      case EOF:    

        if (argc<3 || strcmp(argv[OUTFILE],".")!=0)
          { outvarname[0]="NAME";
            status=writevar(outfile,0,1,outvarname,NULL,DEFAULT);
            if (status!=OK) exiterror(status,argv[0],outfname,0);
          }

        for (i=0;i<nreqvar;i++)
          { dout.field[0]=reqvarname[i];
            status=
              writedata(outfile,0,&dout,1,NULL,NULL,NULL,DEFAULT,DEFAULT,1);
            if (status!=OK) exiterror(status,argv[0],outfname,0);
          }

        break;

      default:
        exiterror(status,argv[0],argv[INFILE],lineno);
    }

  if (infile) fclose(infile);
  fclose(outfile);
  return EXIT_SUCCESS;
}  

