/***********************************************************************
SOFTWARE RELEASE: MarMOT 1.1.1 DATE: 23 January 2015 LICENCE: CeCILL 2.1
***********************************************************************/
/***********************************************************************

SOFTWARE RELEASE: MarMOT 1.1  DATE: 21 November 2013

Copyright Natural Environment Research Council, UK

Author: John Hemmings, National Oceanography Centre 
e-mail: j.hemmings@noc.ac.uk (until 30 Nov 2013) 
Alternative e-mail for author: marmot@wessexenv.mail1.co.uk 

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

***********************************************************************/

/* tabsplit.c - John Hemmings (Oct 2000) */

/* v1.01 - Apr 2003 - MAXFILES increased 
   v1.02 - May 2006 - added NULL pointer check before fclose */

#include "toolbox.h"

#define INFILE 1
#define OUTFILE 2
#define VOFFSET 3
#define MAXFILES 1000

struct filerec {
  char name[FILENAME_MAX+1];
  FILE *ptr;
};


int main (int argc, char *argv[])

{ FILE *infile,*fptr;
  struct filerec outfile[MAXFILES];
  char outfname[FILENAME_MAX+1];
  char varlist[LINE_MAX+1];
  char *invarname[NFIELD_MAX],*byvarname[NFIELD_MAX],*reqvarname[NFIELD_MAX],
       *outvarname[NFIELD_MAX];
  int byvar[NFIELD_MAX],reqvar[NFIELD_MAX];
  static double x[NFIELD_MAX];
  static char fieldlist[LINE_MAX+1];
  static char *field[NFIELD_MAX];
  struct datarec d={x,fieldlist,field,NULL};
  static double x2[NFIELD_MAX];
  static char fieldlist2[LINE_MAX+1];
  static char *field2[NFIELD_MAX];
  struct datarec bycurr={x2,fieldlist2,field2,NULL};
  char sbuff[LINE_MAX+1];
  char *s,*fld;
  int ninvar,nbyvar,byoffset,nreqvar,noutvar,nfiles,status,found,len,i,j,
      ttb,lineno;

  version("1.02",VHDR,argc,argv);

  if (argc<4 || strcmp(argv[VOFFSET],"by")==0 && argc<5)
    { fprintf(stderr,
              "Usage: %s infile outfile_root keyvar1 [keyvar2 ...]\n",argv[0]);
      exit(EXIT_FAILURE);
    }

  for (i=1;i<argc;i++)
    if (strpbrk(argv[i],WHITESPACE)!=NULL) exiterror(BADARG,argv[0],argv[i],0);

  if (strcmp(argv[INFILE],"-")==0) infile=stdin;
  else if ((infile=fopen(argv[INFILE],"r"))==NULL)
    exiterror(OPENINERR,argv[0],argv[INFILE],0);

  if (strcmp(argv[OUTFILE],"-")==0)
    exiterror(OPENOUTERR,argv[0],"-.*",0);

  status=readvar(infile,varlist,invarname,&ninvar,&ttb,&lineno);
  if (status!=OK) exiterror(status,argv[0],argv[INFILE],lineno);

  if (strcmp(argv[VOFFSET],"by")==0) byoffset=VOFFSET+1;
  else byoffset=VOFFSET;
  nbyvar=argc-byoffset;

  if (nbyvar>NFIELD_MAX) exiterror(OUTVARERR,argv[0],NULL,0);
  for (i=0;i<nbyvar;i++) byvarname[i]=argv[byoffset+i];

  if (indexlist(nbyvar,byvarname,ninvar,invarname,byvar)<nbyvar)
    { for (i=0;byvar[i]>=0;i++);
      exiterror(VARERR,argv[0],byvarname[i],0);
    }

  nreqvar=0;
  for (i=0;i<ninvar;i++)
    { j=0;
      found=0;
      while (j<nbyvar && !(found=(byvar[j]==i))) j++;
      if (!found)
        { reqvar[nreqvar]=i;
          reqvarname[nreqvar]=invarname[i];
          nreqvar++;
        }
    }

  noutvar=0;
  for (i=0;i<nbyvar;i++) outvarname[noutvar++]=argv[byoffset+i];
  for (i=0;i<nreqvar;i++) outvarname[noutvar++]=reqvarname[i];

  nfiles=0;

  for (i=0;i<nbyvar;i++) 
    { bycurr.field[i]="";
      bycurr.x[i]=NODATA;
    }

  while ((status=
          readdata(infile,ttb,ninvar,nbyvar,byvar,0,NULL,&d,&lineno))==OK)

    { if (datacomp(nbyvar,&d,byvar,&bycurr,NULL,NUMCOMP)!=0)

	{ copydata(&bycurr,&d,nbyvar,byvar);

          /* Construct output file name for by group */

	  len=strlen(argv[OUTFILE]);
	  if (len>FILENAME_MAX)
	    exiterror(FNAMETRUNC,argv[0],argv[OUTFILE],0);
	  strcpy(outfname,argv[OUTFILE]);
          s=outfname+len;

          for (i=0;i<nbyvar;i++)

	    { fld=bycurr.field[i];
	      if (fld==NULL)
                { sprintf(sbuff,"%.*g",SIGDIG_DEF,bycurr.x[i]);
                  fld=sbuff;
                }
              if (strchr(fld,'.')!=NULL)
	        exiterror(FMTERR,argv[0],argv[INFILE],lineno);
              len+=1+strlen(fld);
	      if (len>FILENAME_MAX)
		exiterror(FNAMETRUNC,argv[0],outfname,0);
              strcpy(s++,".");
	      strcpy(s,fld);
	      s=outfname+len;
	    }

	  if (ttb) ttbname(outfname,outfname);

          /* Check if we already have a file for this group 
             and re-open it if necessary */

          fptr=NULL;
          for (i=0;i<nfiles && fptr==NULL;i++)
            if (strcmp(outfname,outfile[i].name)==0)
              { if (outfile[i].ptr!=NULL) fptr=outfile[i].ptr;
                else
                  { j=0;
                    while (outfile[j].ptr==NULL) j++;
                    fclose(outfile[j].ptr);
                    outfile[j].ptr=NULL;
                    if ((fptr=fopen(outfname,"a"))==NULL)
                      exiterror(OPENOUTERR,argv[0],outfname,0);
                    outfile[i].ptr=fptr;
                  }
              }

	  if (fptr==NULL)

	    { /* Open new file */  

              if (nfiles==MAXFILES) exiterror(FILELIMIT,argv[0],outfname,0);
              if (nfiles>=FOPEN_MAX-1)
                { j=0;
                  while (outfile[j].ptr==NULL) j++;
                  fclose(outfile[j].ptr);
                  outfile[j].ptr=NULL;
                }  
              if ((fptr=fopen(outfname,"w"))==NULL)
                exiterror(OPENOUTERR,argv[0],outfname,0);
              strcpy(outfile[nfiles].name,outfname);
              outfile[nfiles].ptr=fptr;
	      nfiles++;

	      status=writevar(fptr,ttb,noutvar,outvarname,NULL,DEFAULT);
              if (status!=OK) exiterror(status,argv[0],argv[OUTFILE],0);
	    }
        }
 
      status=
	writedata(fptr,ttb,&d,nbyvar,byvar,NULL,NULL,DEFAULT,DEFAULT,0);
      if (status!=OK) exiterror(status,argv[0],argv[OUTFILE],0);
      status=
	writedata(fptr,ttb,&d,nreqvar,reqvar,NULL,NULL,DEFAULT,DEFAULT,1);
      if (status!=OK) exiterror(status,argv[0],argv[OUTFILE],0);
    }

    if (status!=EOF) exiterror(status,argv[0],argv[INFILE],lineno);


  fclose(infile);
  for (i=0;i<nfiles;i++)
    if (outfile[i].ptr) fclose(outfile[i].ptr);
  return EXIT_SUCCESS;
}


