/***********************************************************************
SOFTWARE RELEASE: MarMOT 1.1.1 DATE: 23 January 2015 LICENCE: CeCILL 2.1
***********************************************************************/
/***********************************************************************

SOFTWARE RELEASE: MarMOT 1.1  DATE: 21 November 2013

Copyright Natural Environment Research Council, UK

Author: John Hemmings, National Oceanography Centre 
e-mail: j.hemmings@noc.ac.uk (until 30 Nov 2013) 
Alternative e-mail for author: marmot@wessexenv.mail1.co.uk 

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

***********************************************************************/

/* tabsqz.c - John Hemmings (Oct 2000) */

/* v1.01 - Jul 06 - changed to allow more significant digits than default for
           binary input 
   v1.02 - Nov 2008 - replaced 'mreaddata_special' with 'getdata' to handle
           column data record format + 
           made outfile argument optional 
*/ 


#include "toolbox.h"

#define INFILE 1
#define OUTFILE 2
#define LINESEP -2


int main (int argc, char *argv[])

{ FILE *infile, *outfile;
  struct datarec *dnew=NULL,*d=NULL,*head=NULL;
  char fieldlist[LINE_MAX+1];
  char *field[NFIELD_MAX];
  char varlist[LINE_MAX+1];
  char *varname[NFIELD_MAX];
  double x;
  struct readctrlrec readctrl;
  int maxlen[NFIELD_MAX],sd[NFIELD_MAX],len;
  int nvar,status,wstatus,numerr,i,j,nf,ttb,just=DEFAULT,lineno,count;
  int pagelen=NREC_MAX,pagehdr=0,nspace=1,fwidth=LINE_MAX,sigdig=DEFAULT,
      first=1;

  version("1.02",VHDR,argc,argv);

  if (argc<2)
    { fprintf(stderr,
"Usage: %s infile [outfile [n=n_per_page] [s=col_spacing] [w=max_col_width]\n",
             argv[0]);
      fprintf(stderr," [j=justification] [d=sig_digits]]\n");
      exit(EXIT_FAILURE);
    }

  for (i=1;i<argc;i++)
    if (strpbrk(argv[i],WHITESPACE)!=NULL) exiterror(BADARG,argv[0],argv[i],0);

  if (strcmp(argv[INFILE],"-")==0) infile=stdin;
  else if ((infile=fopen(argv[INFILE],"r"))==NULL)
    exiterror(OPENINERR,argv[0],argv[INFILE],0);

  status=readvar(infile,varlist,varname,&nvar,&ttb,&lineno);
  if (status==OK_INLINE)
    { fclose(infile);
      infile=NULL;
    }
  else if (status!=OK) exiterror(status,argv[0],argv[INFILE],lineno);

  for (i=3;i<argc;i++)
    { if (strncmp(argv[i],"n=",2)==0) 
        { if (sscanf(argv[i]+2,"%d",&pagelen)<1 || pagelen<1) 
            exiterror(BADEXPR,argv[0],argv[i],0); 
          pagehdr=1;
        }
      else if (strncmp(argv[i],"s=",2)==0) 
        { if (*(argv[i]+2)=='-') nspace=LINESEP; 
          else if (sscanf(argv[i]+2,"%d",&nspace)<1)
            exiterror(BADEXPR,argv[0],argv[i],0); 
        }
      else if (strncmp(argv[i],"w=",2)==0) 
        { if (sscanf(argv[i]+2,"%d",&fwidth)<1 || fwidth<0) 
            exiterror(BADEXPR,argv[0],argv[i],0); 
        }
      else if (strncmp(argv[i],"d=",2)==0) 
        { if (sscanf(argv[i]+2,"%d",&sigdig)<1 || sigdig<1)
            exiterror(BADEXPR,argv[0],argv[i],0);
        }
      else if (strncmp(argv[i],"j=",2)==0)
        { if (*(argv[i]+2)=='\0') exiterror(BADEXPR,argv[0],argv[i],0);
          if (strchr("lL",*(argv[i]+2))) just=LEFT; 
          else if (strchr("rR",*(argv[i]+2))) just=RIGHT; 
          else exiterror(BADEXPR,argv[0],argv[i],0); 
        }
      else exiterror(BADEXPR,argv[0],argv[i],0);
    } 

  for (i=0;i<nvar;i++) sd[i]=sigdig;

  if (argc<3 || strcmp(argv[OUTFILE],"-")==0) outfile=stdout;
  else if ((outfile=fopen(argv[OUTFILE],"w"))==NULL)
    exiterror(OPENOUTERR,argv[0],argv[OUTFILE],0);

  readctrl.nkeyvar=0;
  readctrl.keyvar=NULL;
  readctrl.nstrvar=nvar;
  readctrl.strvar=NULL;

  do

    { for (i=0;i<nvar;i++) 
        { maxlen[i]=strlen(varname[i]);
          if (maxlen[i]>fwidth) maxlen[i]=fwidth;
        }

      count=0;
      while (count<pagelen &&
             (status=getdata(nvar,sd,infile,ttb,&readctrl,varname,NULL,&dnew,
                             &lineno))==OK)

        { if (head==NULL) head=dnew;
          else d->next=dnew;
          d=dnew;
          count++;

          for (i=0;i<nvar;i++)
            { len=strlen(d->field[i]);
              if (len>maxlen[i]) maxlen[i]=len;
              if (maxlen[i]>fwidth) maxlen[i]=fwidth;
            }
        }

      if ((pagehdr && count>0) || first) 
        { wstatus=writevar(outfile,0,nvar,varname,maxlen,nspace);
          if (wstatus!=OK) exiterror(wstatus,argv[0],argv[OUTFILE],0);
          first=0;
        }

      if (count>0) 
        { d=head;
          while (d)
            { wstatus=writedata(outfile,0,d,nvar,NULL,maxlen,sd,just,nspace,1);
              if (wstatus!=OK) exiterror(wstatus,argv[0],argv[OUTFILE],0);
              d=d->next;
            }
          if (status==OK && pagehdr) 
            if (fprintf(outfile,"\n")<0) 
              exiterror(wstatus,argv[0],argv[OUTFILE],0);
          freelist(&head);
        }
    }

  while (status==OK);
  if (status!=EOF) exiterror(status,argv[0],argv[INFILE],lineno); 
  
  if (infile) fclose(infile);
  fclose(outfile);
  return EXIT_SUCCESS;
}

