/***********************************************************************
SOFTWARE RELEASE: MarMOT 1.1.1 DATE: 23 January 2015 LICENCE: CeCILL 2.1
***********************************************************************/
/***********************************************************************

SOFTWARE RELEASE: MarMOT 1.1  DATE: 21 November 2013

Copyright Natural Environment Research Council, UK

Author: John Hemmings, National Oceanography Centre 
e-mail: j.hemmings@noc.ac.uk (until 30 Nov 2013) 
Alternative e-mail for author: marmot@wessexenv.mail1.co.uk 

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

***********************************************************************/

/* toolbox.h - John Hemmings (Oct 2000) */

/* v1.01 - Nov 2000 - new function 'outfopen' 
   v1.02 - Jan 2001 - new functions 'validvname' and'isvnamech' + 
                      new error type 'VFMTERR'
   v1.03 - Sep 2001 - NREC_MAX increased
   v1.04 - Nov 2001 - FWIDTH_DEF set to 1
   v1.05 - Nov 2001 - LINE_MAX and NFIELD_MAX increased 
   v1.06 - Apr 2002 - new function 'writefdata' + 
                      col separator defns +
                      NODATA_STR changed from "." to "_" +
                      new error type 'OUTTRUNC' 
   v1.07 - Sep 2003 - LINE_MAX increased 
   v1.08 - Jan 2004 - new error type 'BADDATA' 
   v1.09 - Feb 2005 - new functions 'writelogicalpage', 'setcolwidths',
                      'setwidth' and 'overlaydata'
   v1.10 - May 2005 - Included <ctype.h> for missing 'isalpha' & 'isdigit'
                      declarations
   v1.11 - Jul 2006 - new function 'mreaddata_special'
   v1.12 - Nov 2008 - new status OK_INLINE +
                      new structure 'readctrlrec' +
                      new function 'getdata'
   v1.13 - Feb 2009 - new function 'roundval' +
                      NFIELD_MAX increased
   v1.14 - May 2009 - include <float.h> 
   v1.15 - Apr 2010 - NREC_MAX increased 
   v1.16 - Oct 2011 - LINE_MAX,NFIELD_MAX increased 
   v1.17 - Mar 2013 - defined POW10_EXP_LIM to prevent overflow in 'roundval' 
                      function
*/

#define VHDR "1.17"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include <float.h>

#define EXIT_FAILURE 1
#define EXIT_SUCCESS 0
#define LINE_MAX 50000
#define NFIELD_MAX 5000
#define NREC_MAX 1000000
#define FWIDTH_DEF 1 
#define SIGDIG_DEF 8
#define NSPACE_DEF 1
#define DEFAULT -1
#define NODATA -9e99
#define NODATA_STR "_"
#define WHITESPACE " \t\n\r\v\f"
#define COLSEP_LINE -2
#define COLSEP_COMMA -3
#define COLSEP_TAB -4
#define COLSEP_RTFTAB -5

/* The following system-dependent value is used to prevent overflow in 
   'roundval' function resulting in nans for very small numbers.
   Intended value is floor(DBL_MAX_EXP*log10(FLT_RADIX))
    =308 for DBL_MAX_EXP=1024, FLT_RADIX=2 */

#define POW10_EXP_LIM 308 

enum {NUMCOMP,STRCOMP};
enum {LEFT,RIGHT};
enum {OK,OK_INLINE,BADARG,OPENINERR,OPENOUTERR,FILELIMIT,TRUNC,FLDTRUNC,
      FNAMETRUNC,FMTERR,BADDATA,VFMTERR,BADEXPR,BADNUM,BADFUNC,NOVAR,VARERR,
      NOOUTVAR,OUTVARERR,UNSORTED,DUPLICATE,OUTERR,OUTTRUNC,RECLIMIT,MEMERR,
      SPECIAL};

struct datarec {
  double *x;
  char *fieldlist;
  char **field;
  struct datarec *next;
};

struct readctrlrec {
  int nkeyvar;
  int *keyvar;
  int nstrvar;
  int *strvar;
};

struct sortrec {
  int nvar;
  int var[NFIELD_MAX];
  int sense[NFIELD_MAX];
  int ctype[NFIELD_MAX];
};

void version (char *, char *, int, char *[]);

/* Variable list functions */

int readvar (FILE *, char *, char *[], int *, int *, int *);
int writevar (FILE *, int, int, char *[], int[], int);
int indexlist (int, char *[], int, char *[], int[]);
int validvname (char *);
int isvnamech (int);

/* Data record functions */

int readdata (FILE *, int, int, int, int [], int, int [], struct datarec *,
              int *);
int readdata_special (FILE *, int, int, int, int [], int, int [], int [], 
                       struct datarec *, int *);
int mreaddata (FILE *, int, int, int, int [], int, int [], struct datarec **,
               int *);
int mreaddata_special (FILE *, int, int, int, int [], int, int [], int [], 
                       struct datarec **, int *);
int getdata (int, int [], FILE *, int, struct readctrlrec *, char *[], 
             struct datarec *, struct datarec **, int *);
void keyconv (struct datarec *, int, int []);
void copydata (struct datarec *, struct datarec *, int, int []);
void overlaydata (struct datarec *, struct datarec *, int, int [], int [], 
                  char *);
int writedata (FILE *, int, struct datarec *, int, int [], int [], int [], int,
               int, int);
int writefdata (FILE *, struct datarec *, int, int [], char *[], int [], int,
                int);
int writelogicalpage (FILE *, int, int, char *[], int [], int [], int, int, 
                      int, int, struct datarec []);
void setcolwidths (int, char *[], int, struct datarec [], int [], int, int []);
int datacomp (int, struct datarec *, int [], struct datarec *, int [], int);
int sortcomp (struct datarec *, struct datarec *, struct sortrec *);

/* IO primitives */

int getfieldlist (FILE *, char *, char **, int *);
int putstr (FILE *, char *, int, int);
int putnum (FILE *, double, int, int);
int putrowsep (FILE *, int, int [], int);
int putcolsep (FILE *, int, int);
int getbindata (FILE *, int, double []);
int putbindata (FILE *, int, double []);

/* Miscellaneous utilities */

double roundval (double, int);
double sgetnum (char *, int *);
int sputnum (char *, int, double, int, int);
int setwidth (int, int, int);
FILE *outfopen (char *, char *, char **, int *);
char *ttbname (char *, char *);
void freelist (struct datarec **);
void exiterror (int, char *, char *, int);
