/***********************************************************************
SOFTWARE RELEASE: MarMOT 1.1.1 DATE: 23 January 2015 LICENCE: CeCILL 2.1

Copyright 2015 John C. P. Hemmings 

Author: John Hemmings
e-mail: marmot@wessexenv.mail1.co.uk 

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

***********************************************************************/
/***********************************************************************

 genome.h - John Hemmings - 14 Jan 2015 

 Type definitions and declarations required for using genome-handling 
 functions defined in genome.c 

***********************************************************************/

typedef unsigned int **Population;
typedef unsigned int *Genome;
typedef unsigned int *Parmvec_Int;
typedef unsigned int Genome_Element;
typedef unsigned int Parmvec_Int_Element;

/* Structure to hold the mapping information required to code and decode a 
   parameter vector as a bit string (or 'genome') spanning multiple unsigned 
   integers. Each integer is an element of an array representing the complete 
   genome. Bit positions are filled from left to right starting at the 
   most significant bit of element 1. */

struct codemaprec {
  int nparm;                  /* Number of parameters in parameter vector */
  int nbitsg;                 /* Number of bits representing genome of one
                                 one individual */
  int nbitsge;                /* Number of bits in one genome array element */
  int nge;                    /* Number of array elements for whole genome */
  Parmvec_Int bitselect;      /* Bit positions for each param. within a genome 
                                 element */
  Parmvec_Int bitselect2;     /* Additional bit positions in next element */
  int *bitshift;              /* Shifts for integer representations of 
                                 parameter values to sync. with bit positions */
  int njunkbits;              /* No. of unused bits in last genome element */
  Genome_Element junkbits;    /* Bit positions of any unused part of last 
                                 genome element */
};

extern int genome_nbitslimit; /* Max. no of bits for representing a single 
                                 parameter value */
extern Genome_Element genome_uparmmax;
                              /* Max. integer for representing a parameter 
                                 value */


/* Function Prototypes */

int make_codemap (int nparm, int *);
void clear_codemap ();

Population malloc_population (int);
Genome malloc_genome ();
Parmvec_Int malloc_uparmvec ();
void free_population (Population);
void free_genome (Genome);
void free_uparmvec (Parmvec_Int);

Genome genomecode (Parmvec_Int, Genome);
Parmvec_Int genomedecode (Genome, Parmvec_Int);
Genome genomecreate (int, Genome);
Genome genomecopy (Genome, Genome);
int genomematch (Genome, Genome);
int genomediff (Genome, Genome);
int genomestr (Genome, char *);
Genome crossover_uniform (int, double, Genome, int *);
Genome crossover_point (int, double, Genome, int *);
Genome genomecrossover (Genome, Genome, Genome, Genome);

void write_codemap (FILE *);
void write_population (FILE *, int, Population, double *);
void write_genome (FILE *, Genome, double);
void write_uparmvec (FILE *, Parmvec_Int);
void write_genome_element (FILE *, Genome_Element);


